#!/usr/bin/env awk -f

BEGIN {
    FS=",";

    es_repo_index_name="repo_storage_info"
    es_instance_index_name="storage_summary_info"

    csv_repo_field_name="repository_storage"
    csv_instance_field_name="instance_storage"
    csv_total_fields=0
    
    csv_instance_map["timestamp"]="time"
    csv_instance_map["internal_id"]="instance"
    csv_instance_map["service_id"]="instance_name"
    csv_instance_map["service_type"]=""
    csv_instance_map["storage_type"]=""
    csv_instance_map["storage_directory"]=""
    csv_instance_map["used_space"]="used_size"
    csv_instance_map["free_space"]="free_size"
    csv_instance_map["binaries_count"]=""
    csv_instance_map["binaries_size"]=""
    csv_instance_map["artifacts_size"]="artifact_size"
    csv_instance_map["optimization"]=""
    csv_instance_map["files_count"]="artifact_count"
    csv_instance_map["items_count"]=""
    csv_instance_map["artifacts_count"]=""
    csv_instance_map["total_space"]=""

    csv_repo_map["timestamp"]="time"
    csv_repo_map["internal_id"]="instance"
    csv_repo_map["service_id"]="instance_name"
    csv_repo_map["service_type"]=""
    csv_repo_map["repo_key"]="repo_key"
    csv_repo_map["repo_type"]="repo_type"
    csv_repo_map["folders_count"]=""
    csv_repo_map["files_count"]="artifact_count"
    csv_repo_map["used_space"]="used_size"
    csv_repo_map["items_count"]=""
    csv_repo_map["package_type"]=""
    csv_repo_map["percentage"]=""
   
    es_biggest_number=9999999999999999
    repo_type=""

    write_buffer_str=""
    write_location=ENVIRON["CLOC"] 
    write_buffer_lines=500
    write_lines_per_file=ENVIRON["CLPF"]
    write_chunk=0
    write_empty_buffer="false"
}


{
    if( NR<3 ){
    	init()
    }
    main()
}


function main() {
    if ( NF == csv_total_fields ) {
        if(NR>1){
            write_buffer_str=write_buffer_str""make_data_row()
            if(NR%write_buffer_lines == 0){
                write_to_chunk()
            }
        }        
    }else{
        if(NR>1){
            print "ERROR: Error in parsing file: "FILENAME". See line number " NR". Skipping to next file in list."
            exit; 
        }
        print "ERROR: Field count wrong on line number " NR ". Skipping this line : " $0
    }
}
function make_data_row() {
    row_str=""
    row_str=row_str"{ \"index\": {}}\n"
    if(repo_type==es_instance_index_name){
        row_str=row_str"{\"timestamp\":\""$(+csv_field_map["timestamp"])"\",\"internal_id\":\""$(+csv_field_map["internal_id"])"\",\"service_id\":\""$(+csv_field_map["service_id"])"\",\"service_type\":\""$(+csv_field_map["service_type"])"\",\"storage_type\":\""$(+csv_field_map["storage_type"])"\",\"storage_directory\":\""$(+csv_field_map["storage_directory"])"\",\"total_space\":\""$(+csv_field_map["total_space"])"\",\"used_space\":"convert_bytesto_mb(sprintf( "%.0f", $(+csv_field_map["used_space"]) ))",\"free_space\":"convert_bytesto_mb(sprintf( "%.0f", $(+csv_field_map["free_space"]) ))",\"binaries_count\":\""$(+csv_field_map["binaries_count"])"\",\"binaries_size\":\""$(+csv_field_map["binaries_size"])"\",\"artifacts_size\":"($(+csv_field_map["artifacts_size"])?convert_bytesto_mb(sprintf( "%.0f", $(+csv_field_map["artifacts_size"]) )):0)",\"optimization\":\""$(+csv_field_map["optimization"])"\",\"files_count\":"($(+csv_field_map["files_count"])?$(+csv_field_map["files_count"]):0)",\"items_count\":"($(+csv_field_map["items_count"])?$(+csv_field_map["items_count"]):0)",\"artifacts_count\":"($(+csv_field_map["artifacts_count"])?$(+csv_field_map["artifacts_count"]):0)"}\n"
    }else if(repo_type==es_repo_index_name){
        row_str=row_str"{\"timestamp\":\""$(+csv_field_map["timestamp"])"\",\"service_repo_id\":\""$(+csv_field_map["internal_id"])":"$(+csv_field_map["repo_key"])"\",\"internal_id\":\""$(+csv_field_map["internal_id"])"\",\"service_id\":\""$(+csv_field_map["service_id"])"\",\"service_type\":\""$(+csv_field_map["service_type"])"\",\"repo_key\":\""$(+csv_field_map["repo_key"])"\",\"repo_type\":\""$(+csv_field_map["repo_type"])"\",\"folders_count\":"($(+csv_field_map["folders_count"])?$(+csv_field_map["folders_count"]):0)",\"files_count\":"($(+csv_field_map["files_count"])?$(+csv_field_map["files_count"]):0)",\"used_space\":"($(+csv_field_map["used_space"])?convert_bytesto_mb(sprintf( "%.0f", $(+csv_field_map["used_space"]) )):0)",\"items_count\":"($(+csv_field_map["items_count"])?$(+csv_field_map["items_count"]):0)",\"package_type\":\""$(+csv_field_map["package_type"])"\",\"percentage\":\""$(+csv_field_map["percentage"])"\"}\n"
    }
    return row_str
}

function write_to_chunk(){
    if(write_buffer_str == ""){
        return; 
    }
    #print " write_lines_per_file:"write_lines_per_file"   NR%write_lines_per_file: "NR%write_lines_per_file"  write_empty_buffer: "write_empty_buffer
    if( NR%write_lines_per_file == 0 ){
        write_chunk++
    }
    chunk_file_name=write_location"/"repo_type"/chunk_"my_file_name"_"write_chunk".json"
    wrote_once="true"
    print write_buffer_str >> chunk_file_name
    write_files_list[write_chunk]=write_location"/"repo_type"/chunk_"my_file_name"_"write_chunk".json"
    write_buffer_str=""
}

function init() {
    # header line - read the field names 
    if( NR==1 ){
        #checking whether params are passed as required
        if(ENVIRON["CLPF"] == "" || ENVIRON["CLOC"]=="" || FILENAME==""){
            print "CLPF="ENVIRON["CLPF"]" CLOC="ENVIRON["CLOC"]" FILENAME="ENVIRON["FILENAME"] 
            print "Required values for parsing csv missing!. Exiting" 
            exit;   
        }
        csv_total_fields=NF
        split(FILENAME,fn1,"/")
        split(fn1[length(fn1)],fn2,".")
        my_file_name=fn2[1]

        for(i=1;i<=NF;i++){
            gsub(/\r?\n?$/, "", $i)
            csv_fields[$i]=i
        }
    }
    # first data - read and intepret field mapping
    if( NR==2 ) {
        if( $1==csv_repo_field_name ){
            repo_type=es_repo_index_name
        }else if ( $1==csv_instance_field_name ) {
            repo_type=es_instance_index_name
        }
        map_fields()
		for ( x in csv_field_map ) {
        	#print "key "x"   -- val: "csv_field_map[x]
    	}
    }
}
function convert_bytesto_mb(storage_val){
    storage_val=storage_val/1024/1024
    if( (storage_val>es_biggest_number) ) {
        return es_biggest_number;
    }
    return sprintf( "%.0f", storage_val )
}
function map_fields(){
    for ( x in csv_fields ) {
            #print "fieldmap "x"   -- val: "csv_fields[x]
        }
    
    if(repo_type==es_instance_index_name){
		for ( x in csv_instance_map ) {
            if(csv_instance_map[x] in csv_fields){
			    csv_field_map[x]=csv_fields[csv_instance_map[x]]
            }else {
			    csv_field_map[x]=csv_total_fields+1
            }
		}           
    }else if(repo_type==es_repo_index_name){
        for ( x in csv_repo_map ) {
            if(csv_repo_map[x] in csv_fields){
                csv_field_map[x]=csv_fields[csv_repo_map[x]]
            }else {
                csv_field_map[x]=csv_total_fields+1
            }
        }
    }
}

END{
    write_empty_buffer="true"
    write_to_chunk()
    if(wrote_once=="true"){
        for ( x in write_files_list ) {
           print "√ "write_files_list[x]
        }
    }else{
        print "ERROR: Parsing failed. Check for errors"
    }
    exit;  
}
