#!/usr/bin/env bash

function usage()
{
    echo "Usage: exportInflux.sh <influx_cmd> <influx_dir>"
    exit 1
}

function executeQueryOnInflux()
{
    local request=$1
    local output=$2
    ${influxCmd} -database mission_control -format csv -precision rfc3339 \
        -execute "${request}" > "${influxDir}/${output}"
    if [ $? -eq 0 ]
    then
        echo -e "\nINFO: Successfully stored '${request}' results to ${output}"
    else
        echo -e "\nWARN: Error when executing '${request}'"
    fi
}

function export() {
    executeQueryOnInflux 'select * from hour_data_policy.instance_storage' 'hour_data_instance.csv'
    executeQueryOnInflux 'select * from day_data_policy.instance_storage' 'day_data_instance.csv'
    executeQueryOnInflux 'select * from week_data_policy.instance_storage' 'week_data_instance.csv'
    executeQueryOnInflux 'select * from year_data_policy.instance_storage' 'year_data_instance.csv'
    executeQueryOnInflux 'select * from hour_data_policy.repository_storage' 'hour_data_repository.csv'
    executeQueryOnInflux 'select * from day_data_policy.repository_storage' 'day_data_repository.csv'
    executeQueryOnInflux 'select * from week_data_policy.repository_storage' 'week_data_repository.csv'
    executeQueryOnInflux 'select * from year_data_policy.repository_storage' 'year_data_repository.csv'
}

if [ "$#" -eq 2 ]; then
  influxCmd=$1
  influxDir=$2
else
  usage
fi

export