#!/bin/bash

OFFLINE_MODE=false
AUTOMATION_MODE="false"

# To pull images from a registry which needs login credentials, JFMC_DOCKER_REPO_LOGIN and JFMC_DOCKER_REPO_PASSWORD needs to be set as environment variables
#
# export JFMC_DOCKER_REPO_LOGIN=<username>
# export JFMC_DOCKER_REPO_PASSWORD=<token>
# 
# The above lines can be used to set env variables for this session or add them to ~/.bashrc to set it permanently
# 
# Usage : ./mission-control.sh install

while [[ $# > 1 ]]; do
    case "$1" in
        -u)
            [[ "$JFMC_DOCKER_REPO_LOGIN" ]] || credsChangeMessage
            shift 2
        ;;
        -p)
            [[ "$JFMC_DOCKER_REPO_PASSWORD" ]] || credsChangeMessage
            shift 2
        ;;
        -a | --use-defaults)
            echo "Running in non-interactive mode"
            AUTOMATION_MODE="true"
            shift
        ;;
        -o | --offline)
            echo "Running in offline mode"
            OFFLINE_MODE=true
            shift
        ;;
        * )
            CMD="$CMD $1"
            shift
        ;;
    esac
done

[[ $CMD ]] || CMD="$@"

##################################
############## MAIN ##############
##################################

setUp
createInstallerConfigFile
loadInstallerConfigFile
setupDockerParameters
[[ ! -z "${JFMC_DOCKER_REPO_LOGIN}" && ! -z "${JFMC_DOCKER_REPO_PASSWORD}" ]] && setUpDevDockerParamters || true
setCommonEnvVars

exec docker run --name "jfmc_installer" --rm \
${COMMON_ENV_VARS} ${IMAGE} "${CMD}"

exit $?